// Heuristika.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <iostream>
#include <iomanip>
#include <windows.h> 
#include <conio.h>

using namespace std;

#define SirkaPole	8
#define VyskaPole	8
#define MaxPocetTahu	((SirkaPole * VyskaPole) - 1)

typedef struct {
	int dx;	// zmena v x
	int dy;	// zmena v y
} TahKonem;

TahKonem MozneTahy[8] = { {2, 1}, {1, 2}, {-1, 2}, {-2, 1},
{-2, -1}, {-1, -2}, {1, -2}, {2, -1} };	// osm moznych tahu konem

int Sachovnice[SirkaPole][VyskaPole];

unsigned int dwStart, dwStop; // zacatek a konec trideni

void TiskSachovnice()
{
	for(int i = 0; i < SirkaPole; i++)
	{
		for(int j = 0; j < VyskaPole; j++)
		{
			cout << setw(4) << Sachovnice[i][j];
		}
		cout << endl << endl;
	}
}

void VymazSachovnici()
{
	for(int i = 0; i < SirkaPole; i++)
	{
		for(int j = 0; j < VyskaPole; j++)
		{
			Sachovnice[i][j] = -1;
		}
	}
}

int PocetMoznychTahu(int _PozX, int _PozY)
{
	int iPocet = 0;
	int NoveX, NoveY;

	for(int iTah = 0; iTah < 8; iTah++)
	{
		NoveX = _PozX + MozneTahy[iTah].dx;
		NoveY = _PozY + MozneTahy[iTah].dy;

		if((NoveX >= 0) && (NoveX < SirkaPole) && (NoveY >= 0) && (NoveY < VyskaPole))
		{
			// pole lezi uvnitr sachovnice, jeste overit, zda bylo navstiveno
			if(Sachovnice[NoveX][NoveY] == -1)
			{
				// nebylo jeste navstiveno - pripocteme si mozny tah
				iPocet++;
			}
		}
	}

	return iPocet;
}

void NastavSmery(int _PozX, int _PozY, int *_lpTahy, int *_lpPocetTahu)
{
	int iPocetMoznychTahu[8];	// ukladame sem informaci o tom, kolik je moznych tahu z pole
								// index odpovida tahu
	int NoveX, NoveY;
	
	*_lpPocetTahu = 0;

	// najdeme vsechny tahy z dane pozice, ktere jsou proveditelne a najdeme k nim pocet dosazitelnych poli
	for(int iTah = 0; iTah < 8; iTah++)
	{
		NoveX = _PozX + MozneTahy[iTah].dx;
		NoveY = _PozY + MozneTahy[iTah].dy;

		if((NoveX >= 0) && (NoveX < SirkaPole) && (NoveY >= 0) && (NoveY < VyskaPole))
		{
			// pole lezi uvnitr sachovnice, jeste overit, zda bylo navstiveno
			if(Sachovnice[NoveX][NoveY] == -1)
			{
				// nebylo jeste navstiveno
				_lpTahy[*_lpPocetTahu] = iTah;	// ulozime si index tahu
				iPocetMoznychTahu[*_lpPocetTahu] = PocetMoznychTahu(NoveX, NoveY);	// urcime si pocet tahu z pole
				(*_lpPocetTahu)++;	// pripocteme si mozny tah
			}
		}
	}

	// nyni uz jen setridime tahy podle poctu volnych tahu
	for(int i = 0; i < (*_lpPocetTahu) - 1; i++)
	{
		int k = i;
		for(int j = i + 1; j < (*_lpPocetTahu); j++)
		{
			if(iPocetMoznychTahu[j] < iPocetMoznychTahu[k])
			{
				k = j;	// index nejmensiho pak bude j
			}
		}

		if(k > i)
		{
			// prohodime dva tahy i pocet tahu, aby si obe pole odpovidala
			int iVymena = iPocetMoznychTahu[k];
			iPocetMoznychTahu[k] = iPocetMoznychTahu[i];
			iPocetMoznychTahu[i] = iVymena;
				
			iVymena = _lpTahy[k];
			_lpTahy[k] = _lpTahy[i];
			_lpTahy[i] = iVymena;
		}
	}
}


void UdelejTah(int _PozX, int _PozY, bool *_bTahOK)
{
	int iTah = 0;
	bool bTahOK = false;
	int NoveX, NoveY;
	int iPocetTahu;	// urcuje pocet tahu nalezenych heuristikou
	int iMozneTahy[8];	// udava indexy tahu urcenych heuristikou do pole moznych tahu
	int iCislo = Sachovnice[_PozX][_PozY] + 1;

	// urcime si posloupnost tahu heuristikou
	NastavSmery(_PozX, _PozY, iMozneTahy, &iPocetTahu);
		
	// a nalezene tahy provedeme
	while((iTah < iPocetTahu) && !bTahOK)
	{
		// indexujeme pomoci indexu v pole iMozneTahy
		NoveX = _PozX + MozneTahy[iMozneTahy[iTah]].dx;
		NoveY = _PozY + MozneTahy[iMozneTahy[iTah]].dy;
		iTah++;	// priste zkusime dalsi tah

		// rovnou umistime na pole, tah je urcite proveditelny
		Sachovnice[NoveX][NoveY] = iCislo;

		if(iCislo < MaxPocetTahu)	// pokud tah nebyl posledni, zkus dalsi
		{
			// zkusime dalsi tah
			UdelejTah(NoveX, NoveY, &bTahOK);
					
			if(!bTahOK)
			{
				// pokud tah neni mozny, pak ho zrusime - nastavime na -1
				Sachovnice[NoveX][NoveY] = -1;
			}
		}
		else
		{
			bTahOK = true;
		}
	};

	*_bTahOK = bTahOK;	// vratime hodnotu pres parametr
}

int main(int argc, char* argv[])
{
	int PozX, PozY;

	cout << "Zadej x souradnici kone: ";
	cin >> PozX;
	cout << "Zadej y souradnici kone: ";
	cin >> PozY;

	if((PozX >= 0) && (PozX < SirkaPole) && (PozY >= 0) && (PozY < VyskaPole))
	{
		VymazSachovnici();

		Sachovnice[PozX][PozY] = 0;	// prvni pole

		bool bOK;

		dwStart = GetTickCount();
		UdelejTah(PozX, PozY, &bOK);
		dwStop = GetTickCount();

		// pokud ma bOK hodnotu true, mame zaplnenou sachovnici tahy konem - staci ji vytisknout

		if(bOK)
		{
			TiskSachovnice();
			cout << endl << " Nalezeni cesty trvalo " << dwStop - dwStart << " milisekund" << endl;
		}
		else
		{
			cout << "Cesta konem nebyla nalezena" << endl;
			TiskSachovnice();
		}
	}
	else
	{
		cout << endl << "Zadana pozice lezi mimo hraci pole" << endl;
	}

	char c;
	cin >> c;

	return 0;
}

